@@ -73,7 +73,7 @@ static int xf86ScrnInfoPrivateCount = 0;
 #ifdef XFree86LOADER
 /* Add a pointer to a new DriverRec to xf86DriverList */
 
-void
+_X_EXPORT void
 xf86AddDriver(DriverPtr driver, pointer module, int flags)
 {
     /* Don't add null entries */
@@ -90,14 +90,17 @@ xf86AddDriver(DriverPtr driver, pointer module, int flags)
     if (flags & HaveDriverFuncs)
 	*xf86DriverList[xf86NumDrivers - 1] = *driver;
     else {
-	memcpy(xf86DriverList[xf86NumDrivers - 1], driver, sizeof(DriverRec1));
-	xf86DriverList[xf86NumDrivers - 1]->driverFunc = NULL;
+	(void) memset( xf86DriverList[xf86NumDrivers - 1], 0,
+		       sizeof( DriverRec ) );
+	(void) memcpy( xf86DriverList[xf86NumDrivers - 1], driver, 
+		       sizeof(DriverRec1));
+
     }
     xf86DriverList[xf86NumDrivers - 1]->module = module;
     xf86DriverList[xf86NumDrivers - 1]->refCount = 0;
 }
 
-void
+_X_EXPORT void
 xf86DeleteDriver(int drvIndex)
 {
     if (xf86DriverList[drvIndex]
@@ -111,7 +114,7 @@ xf86DeleteDriver(int drvIndex)
 
 /* Add a pointer to a new InputDriverRec to xf86InputDriverList */
 
-void
+_X_EXPORT void
 xf86AddInputDriver(InputDriverPtr driver, pointer module, int flags)
 {
     /* Don't add null entries */
@@ -140,7 +143,7 @@ xf86DeleteInputDriver(int drvIndex)
     xf86InputDriverList[drvIndex] = NULL;
 }
 
-void
+_X_EXPORT void
 xf86AddModuleInfo(ModuleInfoPtr info, pointer module)
 {
     /* Don't add null entries */
@@ -159,7 +162,7 @@ xf86AddModuleInfo(ModuleInfoPtr info, pointer module)
     xf86ModuleInfoList[xf86NumModuleInfos - 1]->refCount = 0;
 }
 
-void
+_X_EXPORT void
 xf86DeleteModuleInfo(int idx)
 {
     if (xf86ModuleInfoList[idx]) {
@@ -174,7 +177,7 @@ xf86DeleteModuleInfo(int idx)
 
 /* Allocate a new ScrnInfoRec in xf86Screens */
 
-ScrnInfoPtr
+_X_EXPORT ScrnInfoPtr
 xf86AllocateScreen(DriverPtr drv, int flags)
 {
     int i;
@@ -229,7 +232,7 @@ xf86AllocateScreen(DriverPtr drv, int flags)
  * data.  To do this properly may require a driver hook.
  */
 
-void
+_X_EXPORT void
 xf86DeleteScreen(int scrnIndex, int flags)
 {
     ScrnInfoPtr pScrn;
@@ -287,7 +290,7 @@ xf86DeleteScreen(int scrnIndex, int flags)
  * Allocate a private in ScrnInfoRec.
  */
 
-int
+_X_EXPORT int
 xf86AllocateScrnInfoPrivateIndex(void)
 {
     int idx, i;
@@ -308,7 +311,7 @@ xf86AllocateScrnInfoPrivateIndex(void)
 
 /* Allocate a new InputInfoRec and add it to the head xf86InputDevs. */
 
-InputInfoPtr
+_X_EXPORT InputInfoPtr
 xf86AllocateInput(InputDriverPtr drv, int flags)
 {
     InputInfoPtr new;
@@ -334,7 +337,7 @@ xf86AllocateInput(InputDriverPtr drv, int flags)
  * data.  To do this properly may require a driver hook.
  */
 
-void
+_X_EXPORT void
 xf86DeleteInput(InputInfoPtr pInp, int flags)
 {
     InputInfoPtr p;
@@ -374,7 +377,7 @@ xf86DeleteInput(InputInfoPtr pInp, int flags)
     xfree(pInp);
 }
 
-Bool
+_X_EXPORT Bool
 xf86AddPixFormat(ScrnInfoPtr pScrn, int depth, int bpp, int pad)
 {
     int i;
@@ -449,7 +452,7 @@ xf86AddPixFormat(ScrnInfoPtr pScrn, int depth, int bpp, int pad)
 #define GLOBAL_DEFAULT_FBBPP 16
 #endif
 
-Bool
+_X_EXPORT Bool
 xf86SetDepthBpp(ScrnInfoPtr scrp, int depth, int dummy, int fbbpp,
 		int depth24flags)
 {
@@ -777,7 +780,7 @@ xf86SetDepthBpp(ScrnInfoPtr scrp, int depth, int dummy, int fbbpp,
 /*
  * Print out the selected depth and bpp.
  */
-void
+_X_EXPORT void
 xf86PrintDepthBpp(ScrnInfoPtr scrp)
 {
     xf86DrvMsg(scrp->scrnIndex, scrp->depthFrom, "Depth %d, ", scrp->depth);
@@ -788,7 +791,7 @@ xf86PrintDepthBpp(ScrnInfoPtr scrp)
  * xf86SetWeight sets scrp->weight, scrp->mask, scrp->offset, and for depths
  * greater than MAX_PSEUDO_DEPTH also scrp->rgbBits.
  */
-Bool
+_X_EXPORT Bool
 xf86SetWeight(ScrnInfoPtr scrp, rgb weight, rgb mask)
 {
     MessageType weightFrom = X_DEFAULT;
@@ -880,7 +883,7 @@ xf86SetWeight(ScrnInfoPtr scrp, rgb weight, rgb mask)
     return TRUE;
 }
 
-Bool
+_X_EXPORT Bool
 xf86SetDefaultVisual(ScrnInfoPtr scrp, int visual)
 {
     MessageType visualFrom = X_DEFAULT;
@@ -927,7 +930,7 @@ xf86SetDefaultVisual(ScrnInfoPtr scrp, int visual)
 #define SET_GAMMA(g) \
 	(g) > GAMMA_ZERO ? (g) : 1.0
 
-Bool
+_X_EXPORT Bool
 xf86SetGamma(ScrnInfoPtr scrp, Gamma gamma)
 {
     MessageType from = X_DEFAULT;
@@ -981,7 +984,7 @@ xf86SetGamma(ScrnInfoPtr scrp, Gamma gamma)
 #undef MMPERINCH
 #define MMPERINCH 25.4
 
-void
+_X_EXPORT void
 xf86SetDpi(ScrnInfoPtr pScrn, int x, int y)
 {
     MessageType from = X_DEFAULT;
@@ -1078,7 +1081,7 @@ xf86SetDpi(ScrnInfoPtr pScrn, int x, int y)
 #undef MMPERINCH
 
 
-void
+_X_EXPORT void
 xf86SetBlackWhitePixels(ScreenPtr pScreen)
 {
     if (xf86FlipPixels) {
@@ -1251,7 +1254,7 @@ xf86SetRootClip (ScreenPtr pScreen, Bool enable)
  * whether they are switched in or out by keeping track of the root pixmap's
  * private data, and therefore don't need to access pScrnInfo->vtSema.
  */
-void
+_X_EXPORT void
 xf86EnableDisableFBAccess(int scrnIndex, Bool enable)
 {
     ScrnInfoPtr pScrnInfo = xf86Screens[scrnIndex];
@@ -1293,7 +1296,7 @@ xf86EnableDisableFBAccess(int scrnIndex, Bool enable)
 #undef PREFIX_SIZE
 #define PREFIX_SIZE 14
 
-void
+_X_EXPORT void
 xf86VDrvMsgVerb(int scrnIndex, MessageType type, int verb, const char *format,
 		va_list args)
 {
@@ -1320,7 +1323,7 @@ xf86VDrvMsgVerb(int scrnIndex, MessageType type, int verb, const char *format,
 #undef PREFIX_SIZE
 
 /* Print driver messages, with verbose level specified directly */
-void
+_X_EXPORT void
 xf86DrvMsgVerb(int scrnIndex, MessageType type, int verb, const char *format,
 	       ...)
 {
@@ -1332,7 +1335,7 @@ xf86DrvMsgVerb(int scrnIndex, MessageType type, int verb, const char *format,
 }
 
 /* Print driver messages, with verbose level of 1 (default) */
-void
+_X_EXPORT void
 xf86DrvMsg(int scrnIndex, MessageType type, const char *format, ...)
 {
     va_list ap;
@@ -1343,7 +1346,7 @@ xf86DrvMsg(int scrnIndex, MessageType type, const char *format, ...)
 }
 
 /* Print non-driver messages with verbose level specified directly */
-void
+_X_EXPORT void
 xf86MsgVerb(MessageType type, int verb, const char *format, ...)
 {
     va_list ap;
@@ -1354,7 +1357,7 @@ xf86MsgVerb(MessageType type, int verb, const char *format, ...)
 }
 
 /* Print non-driver messages with verbose level of 1 (default) */
-void
+_X_EXPORT void
 xf86Msg(MessageType type, const char *format, ...)
 {
     va_list ap;
@@ -1365,7 +1368,7 @@ xf86Msg(MessageType type, const char *format, ...)
 }
 
 /* Just like ErrorF, but with the verbose level checked */
-void
+_X_EXPORT void
 xf86ErrorFVerb(int verb, const char *format, ...)
 {
     va_list ap;
@@ -1377,7 +1380,7 @@ xf86ErrorFVerb(int verb, const char *format, ...)
 }
 
 /* Like xf86ErrorFVerb, but with an implied verbose level of 1 */
-void
+_X_EXPORT void
 xf86ErrorF(const char *format, ...)
 {
     va_list ap;
@@ -1429,7 +1432,7 @@ xf86CloseLog()
  * Drivers can use these for using their own SymTabRecs.
  */
 
-const char *
+_X_EXPORT const char *
 xf86TokenToString(SymTabPtr table, int token)
 {
     int i;
@@ -1443,7 +1446,7 @@ xf86TokenToString(SymTabPtr table, int token)
 	return(table[i].name);
 }
 
-int
+_X_EXPORT int
 xf86StringToToken(SymTabPtr table, const char *string)
 {
     int i;
@@ -1460,7 +1463,7 @@ xf86StringToToken(SymTabPtr table, const char *string)
 /*
  * helper to display the clocks found on a card
  */
-void
+_X_EXPORT void
 xf86ShowClocks(ScrnInfoPtr scrp, MessageType from)
 {
     int j;
@@ -1484,7 +1487,7 @@ xf86ShowClocks(ScrnInfoPtr scrp, MessageType from)
  * XXX This makes assumptions about the line width, etc.  Maybe we could
  * use a more general "pretty print" function for messages.
  */
-void
+_X_EXPORT void
 xf86PrintChipsets(const char *drvname, const char *drvmsg, SymTabPtr chips)
 {
     int len, i;
@@ -1513,7 +1516,7 @@ xf86PrintChipsets(const char *drvname, const char *drvmsg, SymTabPtr chips)
 #define MAXDRIVERS 64	/* A >hack<, to be sure ... */
 
 
-int
+_X_EXPORT int
 xf86MatchDevice(const char *drivername, GDevPtr **sectlist)
 {
     GDevPtr       gdp, *pgdp = NULL;
@@ -1596,103 +1599,162 @@ struct Inst {
     int 		screen;
 };
 
-int
+
+/**
+ * Find set of unclaimed devices matching a given vendor ID.
+ * 
+ * Used by drivers to find as yet unclaimed devices matching the specified
+ * vendor ID.
+ *
+ * \param driverName     Name of the driver.  This is used to find Device
+ *                       sections in the config file.
+ * \param vendorID       PCI vendor ID of associated devices.  If zero, then
+ *                       the true vendor ID must be encoded in the \c PCIid
+ *                       fields of the \c PCIchipsets entries.
+ * \param chipsets       Symbol table used to associate chipset names with
+ *                       PCI IDs.
+ * \param devList        List of Device sections parsed from the config file.
+ * \param numDevs        Number of entries in \c devList.
+ * \param drvp           Pointer the driver's control structure.
+ * \param foundEntities  Returned list of entity indicies associated with the
+ *                       driver.
+ * 
+ * \returns
+ * The number of elements in returned in \c foundEntities on success or zero
+ * on failure.
+ *
+ * \todo
+ * This function does a bit more than short description says.  Fill in some
+ * more of the details of its operation.
+ *
+ * \todo
+ * The \c driverName parameter is redundant.  It is the same as
+ * \c DriverRec::driverName.  In a future version of this function, remove
+ * that parameter.
+ */
+_X_EXPORT int
 xf86MatchPciInstances(const char *driverName, int vendorID, 
 		      SymTabPtr chipsets, PciChipsets *PCIchipsets,
 		      GDevPtr *devList, int numDevs, DriverPtr drvp,
 		      int **foundEntities)
 {
     int i,j;
-    MessageType from;
     pciVideoPtr pPci, *ppPci;
-    struct Inst {
-	pciVideoPtr	pci;
-	GDevPtr		dev;
-	Bool		foundHW;  /* PCIid in list of supported chipsets */
-	Bool		claimed;  /* BusID matches with a device section */
-        int             chip;
-        int		screen;
-    } *instances = NULL;
+    struct Inst *instances = NULL;
     int numClaimedInstances = 0;
     int allocatedInstances = 0;
     int numFound = 0;
     SymTabRec *c;
     PciChipsets *id;
-    GDevPtr devBus = NULL;
-    GDevPtr dev = NULL;
     int *retEntities = NULL;
 
     *foundEntities = NULL;
 
-    if (vendorID == 0) {
-	for (ppPci = xf86PciVideoInfo; *ppPci != NULL; ppPci++) {
-	    Bool foundVendor = FALSE;
-	    for (id = PCIchipsets; id->PCIid != -1; id++) {
-	        if ( (((id->PCIid & 0xFFFF0000) >> 16) == (*ppPci)->vendor)) {
-		    if (!foundVendor) {
-	                ++allocatedInstances;
-			instances = xnfrealloc(instances,
-				     allocatedInstances * sizeof(struct Inst));
-			instances[allocatedInstances - 1].pci = *ppPci;
-			instances[allocatedInstances - 1].dev = NULL;
-			instances[allocatedInstances - 1].claimed = FALSE;
-			instances[allocatedInstances - 1].foundHW = FALSE;
-			instances[allocatedInstances - 1].screen = 0;
-			foundVendor = TRUE;
-		    } 
-		    if ((id->PCIid & 0x0000FFFF) == (*ppPci)->chipType) {
-	               instances[allocatedInstances - 1].foundHW = TRUE;
-		       instances[allocatedInstances - 1].chip = id->numChipset;
-		       numFound++;
-		    }
-		}
-	    }
+
+    /* Each PCI device will contribute at least one entry.  Each device
+     * section can contribute at most one entry.  The sum of the two is
+     * guaranteed to be larger than the maximum possible number of entries.
+     * Do this calculation and memory allocation once now to eliminate the
+     * need for realloc calls inside the loop.
+     */
+    if ( !xf86DoProbe && !(xf86DoConfigure && xf86DoConfigurePass1) ) {
+	unsigned max_entries = numDevs;
+	for (ppPci = xf86PciVideoInfo ; *ppPci != NULL ; ppPci++) {
+	    max_entries++;
 	}
-    } else if (vendorID == PCI_VENDOR_GENERIC) {
-	for (ppPci = xf86PciVideoInfo; *ppPci != NULL; ppPci++) {
-	    for (id = PCIchipsets; id->PCIid != -1; id++) {
-		if (id->PCIid == xf86CheckPciGAType(*ppPci)) {
+
+	instances = xnfalloc( max_entries * sizeof(struct Inst) );
+    }
+
+    for (ppPci = xf86PciVideoInfo; *ppPci != NULL; ppPci++) {
+	unsigned device_class = ((*ppPci)->class << 16)
+	    | ((*ppPci)->subclass << 8) | ((*ppPci)->interface);
+	Bool foundVendor = FALSE;
+
+
+	pPci = *ppPci;
+
+	/* Convert the pre-PCI 2.0 device class for a VGA adapter to the
+	 * 2.0 version of the same class.
+	 */
+	if ( device_class == 0x00000101 ) {
+	    device_class = 0x00030000;
+	}
+
+
+	/* Find PCI devices that match the given vendor ID.  The vendor ID is
+	 * either specified explicitly as a parameter to the function or
+	 * implicitly encoded in the high bits of id->PCIid.
+	 *
+	 * The first device with a matching vendor is recorded, even if the
+	 * device ID doesn't match.  This is done because the Device section
+	 * in the xorg.conf file can over-ride the device ID.  A matching PCI
+	 * ID might not be found now, but after the device ID over-ride is
+	 * applied there /might/ be a match.
+	 */
+	for (id = PCIchipsets; id->PCIid != -1; id++) {
+	    const unsigned vendor_id = ((id->PCIid & 0xFFFF0000) >> 16)
+		| vendorID;
+	    const unsigned device_id = (id->PCIid & 0x0000FFFF);
+	    const unsigned match_class = 0x00030000 | id->PCIid;
+
+	    if ( (vendor_id == pPci->vendor)
+		 || ((vendorID == PCI_VENDOR_GENERIC) && (match_class == device_class)) ) {
+		if ( !foundVendor && (instances != NULL) ) {
 		    ++allocatedInstances;
-		    instances = xnfrealloc(instances,
-				  allocatedInstances * sizeof(struct Inst));
 		    instances[allocatedInstances - 1].pci = *ppPci;
 		    instances[allocatedInstances - 1].dev = NULL;
 		    instances[allocatedInstances - 1].claimed = FALSE;
-		    instances[allocatedInstances - 1].foundHW = TRUE;
-		    instances[allocatedInstances - 1].chip = id->numChipset;
+		    instances[allocatedInstances - 1].foundHW = FALSE;
 		    instances[allocatedInstances - 1].screen = 0;
-		    numFound++;
+		    foundVendor = TRUE;
 		}
-	    }
-	}
-    } else {
-	/* Find PCI devices that match the given vendor ID */
-	for (ppPci = xf86PciVideoInfo; (ppPci != NULL)
-	       && (*ppPci != NULL); ppPci++) {
-	    if ((*ppPci)->vendor == vendorID) {
-		++allocatedInstances;
-		instances = xnfrealloc(instances,
-			      allocatedInstances * sizeof(struct Inst));
-		instances[allocatedInstances - 1].pci = *ppPci;
-		instances[allocatedInstances - 1].dev = NULL;
-		instances[allocatedInstances - 1].claimed = FALSE;
-		instances[allocatedInstances - 1].foundHW = FALSE;
-	        instances[allocatedInstances - 1].screen = 0;
-
-		/* Check if the chip type is listed in the chipsets table */
-		for (id = PCIchipsets; id->PCIid != -1; id++) {
-		    if (id->PCIid == (*ppPci)->chipType) {
-			instances[allocatedInstances - 1].chip
-			    = id->numChipset;
+		if ( (device_id == pPci->chipType)
+		     || ((vendorID == PCI_VENDOR_GENERIC) 
+			 && (match_class == device_class)) ) {
+		    if ( instances != NULL ) {
 			instances[allocatedInstances - 1].foundHW = TRUE;
-			numFound++;
-			break;
+			instances[allocatedInstances - 1].chip = id->numChipset;
 		    }
+
+
+		    if ( xf86DoConfigure && xf86DoConfigurePass1 ) {
+			if ( xf86CheckPciSlot(pPci->bus, pPci->device,
+					      pPci->func) ) {
+			    GDevPtr pGDev = 
+			      xf86AddDeviceToConfigure( drvp->driverName,
+							pPci, -1 );
+			    if (pGDev) {
+				/* After configure pass 1, chipID and chipRev
+				 * are treated as over-rides, so clobber them
+				 * here.
+				 */
+				pGDev->chipID = -1;
+				pGDev->chipRev = -1;
+			    }
+
+			    numFound++;
+			}
+		    }
+		    else {
+			numFound++;
+		    }
+
+		    break;
 		}
 	    }
 	}
     }
 
+
+    /* In "probe only" or "configure" mode (signaled by instances being NULL),
+     * our work is done.  Return the number of detected devices.
+     */
+    if ( instances == NULL ) {
+	return numFound;
+    }
+
+
     /*
      * This may be debatable, but if no PCI devices with a matching vendor
      * type is found, return zero now.  It is probably not desirable to
@@ -1703,34 +1765,6 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 	return 0;
     }
 
-    if (xf86DoProbe) {
-	xfree(instances);
-	return numFound;
-    }
-
-    if (xf86DoConfigure && xf86DoConfigurePass1) {
-	GDevPtr pGDev;
-	int actualcards = 0;
-	for (i = 0; i < allocatedInstances; i++) {
-	    pPci = instances[i].pci;
-	    if (instances[i].foundHW) {
-		if (!xf86CheckPciSlot(pPci->bus, pPci->device, pPci->func))
-		    continue;
-		actualcards++;
-	    	pGDev = xf86AddDeviceToConfigure(drvp->driverName,
-						 instances[i].pci, -1);
-		if (pGDev) {
-		   /*
-		    * XF86Match???Instances() treat chipID and chipRev as
-		    * overrides, so clobber them here.
-		    */
-		   pGDev->chipID = pGDev->chipRev = -1;
-		}
-	    }
-	}
-	xfree(instances);
-	return actualcards;
-    }
 
 #ifdef DEBUG
     ErrorF("%s instances found: %d\n", driverName, allocatedInstances);
@@ -1753,9 +1787,6 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
 					    pPci->device,
 					    pPci->func)) {
 		    allocatedInstances++;
-		    instances = xnfrealloc(instances,
-					   allocatedInstances * 
-					   sizeof(struct Inst));
 		    instances[allocatedInstances - 1] = instances[i];
 		    instances[allocatedInstances - 1].screen =
 		      				devList[j]->screen;
@@ -1767,9 +1798,10 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
     }
 
     for (i = 0; i < allocatedInstances; i++) {
+	GDevPtr dev = NULL;
+	GDevPtr devBus = NULL;
+
 	pPci = instances[i].pci;
-	devBus = NULL;
-	dev = NULL;
 	for (j = 0; j < numDevs; j++) {
 	    if (devList[j]->busID && *devList[j]->busID) {
 		if (xf86ComparePciBusString(devList[j]->busID, pPci->bus,
@@ -1825,10 +1857,11 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
      * If chipset is not valid ignore BusSlot completely.
      */
     for (i = 0; i < allocatedInstances && numClaimedInstances > 0; i++) {
+	MessageType from = X_PROBED;
+
 	if (!instances[i].claimed) {
 	    continue;
 	}
-	from = X_PROBED;
 	if (instances[i].dev->chipset) {
 	    for (c = chipsets; c->token >= 0; c++) {
 		if (xf86NameCmp(c->name, instances[i].dev->chipset) == 0)
@@ -1958,7 +1991,7 @@ xf86MatchPciInstances(const char *driverName, int vendorID,
     return numFound;
 }
 
-int
+_X_EXPORT int
 xf86MatchIsaInstances(const char *driverName, SymTabPtr chipsets,
 		      IsaChipsets *ISAchipsets, DriverPtr drvp,
 		      FindIsaDevProc FindIsaDevice, GDevPtr *devList,
@@ -2063,7 +2096,7 @@ xf86MatchIsaInstances(const char *driverName, SymTabPtr chipsets,
 /*
  * xf86GetClocks -- get the dot-clocks via a BIG BAD hack ...
  */
-void
+_X_EXPORT void
 xf86GetClocks(ScrnInfoPtr pScrn, int num, Bool (*ClockFunc)(ScrnInfoPtr, int),
 	      void (*ProtectRegs)(ScrnInfoPtr, Bool),
 	      void (*BlankScreen)(ScrnInfoPtr, Bool), IOADDRESS vertsyncreg,
@@ -2164,7 +2197,7 @@ finish:
     (*ClockFunc)(pScrn, CLK_REG_RESTORE);
 }
 
-void
+_X_EXPORT void
 xf86SetPriority(Bool up)
 {
     static int saved_nice;
@@ -2188,7 +2221,7 @@ xf86SetPriority(Bool up)
     }
 }
 
-const char *
+_X_EXPORT const char *
 xf86GetVisualName(int visual)
 {
     if (visual < 0 || visual > DirectColor)
@@ -2198,62 +2231,62 @@ xf86GetVisualName(int visual)
 }
 
 
-int
+_X_EXPORT int
 xf86GetVerbosity()
 {
     return max(xf86Verbose, xf86LogVerbose);
 }
 
-Pix24Flags
+_X_EXPORT Pix24Flags
 xf86GetPix24()
 {
     return xf86Info.pixmap24;
 }
 
 
-int
+_X_EXPORT int
 xf86GetDepth()
 {
     return xf86Depth;
 }
 
 
-rgb
+_X_EXPORT rgb
 xf86GetWeight()
 {
     return xf86Weight;
 }
 
 
-Gamma
+_X_EXPORT Gamma
 xf86GetGamma()
 {
     return xf86Gamma;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86GetFlipPixels()
 {
     return xf86FlipPixels;
 }
 
 
-const char *
+_X_EXPORT const char *
 xf86GetServerName()
 {
     return xf86ServerName;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86ServerIsExiting()
 {
     return (dispatchException & DE_TERMINATE) == DE_TERMINATE;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86ServerIsResetting()
 {
     return xf86Resetting;
@@ -2267,62 +2300,62 @@ xf86ServerIsInitialising()
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86ServerIsOnlyDetecting(void)
 {
     return xf86DoProbe || xf86DoConfigure;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86ServerIsOnlyProbing(void)
 {
     return xf86ProbeOnly;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86CaughtSignal()
 {
     return xf86Info.caughtSignal;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86GetVidModeAllowNonLocal()
 {
     return xf86Info.vidModeAllowNonLocal;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86GetVidModeEnabled()
 {
     return xf86Info.vidModeEnabled;
 }
 
-Bool
+_X_EXPORT Bool
 xf86GetModInDevAllowNonLocal()
 {
     return xf86Info.miscModInDevAllowNonLocal;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86GetModInDevEnabled()
 {
     return xf86Info.miscModInDevEnabled;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86GetAllowMouseOpenFail()
 {
     return xf86Info.allowMouseOpenFail;
 }
 
 
-Bool
+_X_EXPORT Bool
 xf86IsPc98()
 {
 #if defined(i386) || defined(__i386__)
@@ -2332,20 +2365,20 @@ xf86IsPc98()
 #endif
 }
 
-void
+_X_EXPORT void
 xf86DisableRandR()
 {
     xf86Info.disableRandR = TRUE;
     xf86Info.randRFrom = X_PROBED;
 }
 
-CARD32
+_X_EXPORT CARD32
 xf86GetVersion()
 {
     return XF86_VERSION_CURRENT;
 }
 
-CARD32
+_X_EXPORT CARD32
 xf86GetModuleVersion(pointer module)
 {
 #ifdef XFree86LOADER
@@ -2355,7 +2388,7 @@ xf86GetModuleVersion(pointer module)
 #endif
 }
 
-pointer
+_X_EXPORT pointer
 xf86LoadDrvSubModule(DriverPtr drv, const char *name)
 {
 #ifdef XFree86LOADER
@@ -2372,7 +2405,7 @@ xf86LoadDrvSubModule(DriverPtr drv, const char *name)
 #endif
 }
 
-pointer
+_X_EXPORT pointer
 xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 {
 #ifdef XFree86LOADER
@@ -2392,7 +2425,7 @@ xf86LoadSubModule(ScrnInfoPtr pScrn, const char *name)
 /*
  * xf86LoadOneModule loads a single module.
  */             
-pointer
+_X_EXPORT pointer
 xf86LoadOneModule(char *name, pointer opt)
 {
 #ifdef XFree86LOADER
@@ -2430,7 +2463,7 @@ xf86LoadOneModule(char *name, pointer opt)
     return mod;
 }
 
-void
+_X_EXPORT void
 xf86UnloadSubModule(pointer mod)
 {
     /*
@@ -2442,7 +2475,7 @@ xf86UnloadSubModule(pointer mod)
 #endif
 }
 
-Bool
+_X_EXPORT Bool
 xf86LoaderCheckSymbol(const char *name)
 {
 #ifdef XFree86LOADER
@@ -2452,7 +2485,7 @@ xf86LoaderCheckSymbol(const char *name)
 #endif
 }
 
-void
+_X_EXPORT void
 xf86LoaderReqSymLists(const char **list0, ...)
 {
 #ifdef XFree86LOADER
@@ -2464,7 +2497,7 @@ xf86LoaderReqSymLists(const char **list0, ...)
 #endif
 }
 
-void
+_X_EXPORT void
 xf86LoaderReqSymbols(const char *sym0, ...)
 {
 #ifdef XFree86LOADER
@@ -2476,7 +2509,7 @@ xf86LoaderReqSymbols(const char *sym0, ...)
 #endif
 }
 
-void
+_X_EXPORT void
 xf86LoaderRefSymLists(const char **list0, ...)
 {
 #ifdef XFree86LOADER
@@ -2488,7 +2521,7 @@ xf86LoaderRefSymLists(const char **list0, ...)
 #endif
 }
 
-void
+_X_EXPORT void
 xf86LoaderRefSymbols(const char *sym0, ...)
 {
 #ifdef XFree86LOADER
@@ -2510,7 +2543,7 @@ static const OptionInfoRec BSOptions[] = {
    { -1,                   NULL,           OPTV_NONE,    {0}, FALSE }
 };
 
-void 
+_X_EXPORT void 
 xf86SetBackingStore(ScreenPtr pScreen)
 {
     Bool useBS = FALSE;
@@ -2550,7 +2583,7 @@ static const OptionInfoRec SMOptions[] = {
    { -1,                   NULL,           OPTV_NONE,    {0}, FALSE }
 };
 
-void 
+_X_EXPORT void 
 xf86SetSilkenMouse (ScreenPtr pScreen)
 {
     Bool useSM = TRUE;
@@ -2591,7 +2624,7 @@ xf86SetSilkenMouse (ScreenPtr pScreen)
 
 /* Wrote this function for the PM2 Xv driver, preliminary. */
 
-pointer
+_X_EXPORT pointer
 xf86FindXvOptions(int scrnIndex, int adaptor_index, char *port_name,
 		  char **adaptor_name, pointer *adaptor_options)
 {
@@ -2627,7 +2660,7 @@ xf86FindXvOptions(int scrnIndex, int adaptor_index, char *port_name,
  * special enter/leave functions and their private scratch area to
  * this entity, take the dog for a walk...
  */
-ScrnInfoPtr
+_X_EXPORT ScrnInfoPtr
 xf86ConfigIsaEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
 			  IsaChipsets *i_chip, resList res, EntityProc init,
 			  EntityProc enter, EntityProc leave, pointer private)
@@ -2664,7 +2697,7 @@ xf86ConfigIsaEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     return pScrn;
 }
 
-ScrnInfoPtr
+_X_EXPORT ScrnInfoPtr
 xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
 			  PciChipsets *p_chip, resList res, EntityProc init,
 			  EntityProc enter, EntityProc leave, pointer private)
@@ -2707,7 +2740,7 @@ xf86ConfigPciEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
     return pScrn;
 }
 
-ScrnInfoPtr
+_X_EXPORT ScrnInfoPtr
 xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
 		   EntityProc init, EntityProc enter, EntityProc leave, 
 		   pointer private)
@@ -2740,7 +2773,7 @@ xf86ConfigFbEntity(ScrnInfoPtr pScrn, int scrnFlag, int entityIndex,
  *             are obsolete functions. They the are likely to be removed
  *             Don't use!
  */
-Bool
+_X_EXPORT Bool
 xf86ConfigActiveIsaEntity(ScrnInfoPtr pScrn, int entityIndex,
                           IsaChipsets *i_chip, resList res, EntityProc init,
                           EntityProc enter, EntityProc leave, pointer private)
@@ -2770,7 +2803,7 @@ xf86ConfigActiveIsaEntity(ScrnInfoPtr pScrn, int entityIndex,
     return TRUE;
 }
  
-Bool
+_X_EXPORT Bool
 xf86ConfigActivePciEntity(ScrnInfoPtr pScrn, int entityIndex,
                           PciChipsets *p_chip, resList res, EntityProc init,
                           EntityProc enter, EntityProc leave, pointer private)
@@ -2807,7 +2840,7 @@ xf86ConfigActivePciEntity(ScrnInfoPtr pScrn, int entityIndex,
  * assigned to a screen before it will be removed. If p_pci(p_isa) is
  * non-NULL all static resources listed there will be registered.
  */
-void
+_X_EXPORT void
 xf86ConfigPciEntityInactive(EntityInfoPtr pEnt, PciChipsets *p_chip,
 			    resList res, EntityProc init, EntityProc enter,
 			    EntityProc leave, pointer private)
@@ -2829,7 +2862,7 @@ xf86ConfigPciEntityInactive(EntityInfoPtr pEnt, PciChipsets *p_chip,
     xf86SetEntityFuncs(pEnt->index,init,enter,leave,private);
 }
 
-void
+_X_EXPORT void
 xf86ConfigIsaEntityInactive(EntityInfoPtr pEnt, IsaChipsets *i_chip,
 			    resList res, EntityProc init, EntityProc enter,
 			    EntityProc leave, pointer private)
@@ -2862,7 +2895,7 @@ xf86ConfigFbEntityInactive(EntityInfoPtr pEnt, EntityProc init,
     xf86SetEntityFuncs(pEnt->index,init,enter,leave,private);
 }
 
-Bool
+_X_EXPORT Bool
 xf86IsScreenPrimary(int scrnIndex)
 {
     ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
@@ -2875,7 +2908,7 @@ xf86IsScreenPrimary(int scrnIndex)
     return FALSE;
 }
 
-int
+_X_EXPORT int
 xf86RegisterRootWindowProperty(int ScrnIndex, Atom property, Atom type,
 			       int format, unsigned long len, pointer value )
 {
@@ -2966,7 +2999,7 @@ xf86RegisterRootWindowProperty(int ScrnIndex, Atom property, Atom type,
     return(Success);    
 }
 
-Bool
+_X_EXPORT Bool
 xf86IsUnblank(int mode)
 {
     switch(mode) {
